==> TuxGuitar-0.2.tar.gz: TuxGuitar-0.2/src/org/herac/tuxguitar/gui/tab/Caret.java <==
/*
 * Created on 30-nov-2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package org.herac.tuxguitar.gui.tab;

import java.util.Iterator;
import java.util.List;

import org.eclipse.swt.graphics.GC;
import org.herac.tuxguitar.song.managers.SongManager;
import org.herac.tuxguitar.song.models.Duration;
import org.herac.tuxguitar.song.models.InstrumentString;
import org.herac.tuxguitar.song.models.Measure;
import org.herac.tuxguitar.song.models.SongTrack;

/**
 * @author julian
 * 
 * TODO To change the template for this generated type comment go to Window - Preferences - Java - Code Style - Code Templates
 */
public class Caret {
    private Tablature tablature;
    private SongManager songManager;
    private SongCoords songCoords;
    private SongTrackCoords selectedTrack;
    private MeasureCoords selectedMeasure;
    private MeasureComponent selectedComponent;
    private Duration selectedDuration;
    private int posX;
    private int posY;
    private long position;
    private int string;
    private boolean changes;
    
    public Caret(Tablature tablature,SongManager songManager, SongCoords songCoords) {
        this.tablature = tablature;
        this.songManager = songManager;
        this.songCoords = songCoords;
        this.selectedDuration = new Duration(Duration.QUARTER);
        this.string = 1;
        this.changes = false;
    }

    private void setSelectedMeasureCoords(MeasureCoords selectedMeasure) {
        if (this.selectedMeasure != null) {
            this.selectedMeasure.setCaret(null);
        }
        this.selectedMeasure = selectedMeasure;
        if (this.selectedMeasure != null) {
            this.selectedMeasure.setCaret(this);
        }
    }

    public synchronized void update(Measure measure) {
        if(measure != null){
        SongTrack track = this.songManager.getTrack(measure);
        SongTrackCoords trackCoords = songCoords.getTrack(track);
        if(trackCoords != null){
        MeasureCoords measureCoords = trackCoords.getMeasure(measure);

        MeasureComponent component = null;
        if (getSelectedComponent() == null) {
            component = measureCoords.getFirstComponent();
        } else {
            if (getSelectedComponent() instanceof NoteCoords) {
                NoteCoords noteCoords = (NoteCoords) getSelectedComponent();
                component = measureCoords.getComponent(noteCoords.getStart(), noteCoords.getNote().getString());
            } else {
                component = measureCoords.getComponent(getSelectedComponent().getStart());
            }
        }
        if (component == null) {
            component = measureCoords.getFirstComponent();
        }

        moveTo(trackCoords, measureCoords, component);
        }
        }
    }

    public void paintCaret(GC gc, int fromX, int fromY) {
        if (selectedMeasure != null && selectedComponent != null) {
            long start = this.selectedComponent.getStart();
            this.posX = fromX + TablatureUtil.getStartPosition(selectedMeasure.getMeasure(), start, selectedMeasure.getQuarterSpan()) + 4;
            this.posY = fromY + this.string * SongTrackCoords.DEFAULT_STRING_SPAN - 7;
            
            gc.drawRectangle(this.posX, this.posY, 13, 15);            
        }
        
    }

    public void setStringNumber(int string){
        this.string = string;
    }
    
    public boolean moveRight() {        
        if (this.selectedComponent != null) {
            MeasureComponent nextComponent = selectedMeasure.getNextComponent(this.selectedComponent);
            if (nextComponent != null) {
                this.selectedComponent = nextComponent;
            } else {
                //si no habia mas componentes. busco el siguiente compas
                MeasureCoords nextMeasureCoords = this.selectedTrack.getNextMeasure(this.selectedMeasure);
                if (nextMeasureCoords == null) {
                    return false;
                }else{
                    this.setSelectedMeasureCoords(nextMeasureCoords);
                    this.selectedComponent = this.selectedMeasure.getFirstComponent();
                }
            }
            this.updatePosition();
            this.updateDuration();  
            
            this.setChanges(true);
        }
        return true;
    }

    public void moveLeft() {
        if (this.selectedComponent != null) {
            MeasureComponent prevComponent = selectedMeasure.getPreviousComponent(this.selectedComponent);
            if (prevComponent != null) {
                this.selectedComponent = prevComponent;
            } else {
                //si no habia mas componentes. busco el compas anterior
                MeasureCoords prevMeasureCoords = this.selectedTrack.getPrevMeasure(this.selectedMeasure);
                if (prevMeasureCoords != null) {
                    this.setSelectedMeasureCoords(prevMeasureCoords);
                    this.selectedComponent = this.selectedMeasure.getLastComponent();
                }
            }
            this.updatePosition();
            this.updateDuration();
            
            this.setChanges(true);
        }
    }

    /**
     * Luego de mover el Caret. cambia la duracion seleccionada por la del componente. solo si lo que resta del compas no esta vacio
     */
    private void updateDuration() {
        if (this.selectedComponent != null) {
            boolean hasNotes = false;
            Iterator it = getMeasureCoords().getComponentsBeforeEnd(getSelectedComponent().getStart()).iterator();
            while (it.hasNext()) {
                MeasureComponent component = (MeasureComponent) it.next();
                if (component instanceof NoteCoords) {
                    hasNotes = true;
                    break;
                }
            }
            if (hasNotes) {
                if(this.selectedComponent instanceof SilenceCoords){
                    long length = this.selectedComponent.getDuration().getTime();
                    
                    MeasureComponent nextComponent = getMeasureCoords().getNextComponent(this.selectedComponent);
                    while(nextComponent != null && nextComponent instanceof SilenceCoords){
                        length += nextComponent.getDuration().getTime();                        
                        nextComponent = getMeasureCoords().getNextComponent(nextComponent);
                    }
                    
                    if(this.selectedDuration.getTime() > length){
                        this.selectedDuration = (Duration) this.selectedComponent.getDuration().clone();   
                    }
                }else{
                    this.selectedDuration = (Duration) this.selectedComponent.getDuration().clone();
                }
                
                //this.selectedDuration = (Duration) this.selectedComponent.getDuration().clone();                
            }
        }
    }

    public void moveUp() {
        changeString(-1);
    }

    public void moveDown() {
        changeString(1);
    }

    private void changeString(int i) {
        int nextString = this.string + i;
        int trackStrings = this.selectedTrack.getTrack().getStrings().size();
        if (nextString < 1) {
            nextString = trackStrings;
        } else if (nextString > trackStrings) {
            nextString = 1;
        }
        this.string = nextString;
    }

    public long getPosition() {
        return this.position;
    }

    public SongCoords getSongCoords() {
        return this.songCoords;
    }

    public MeasureCoords getMeasureCoords() {
        return this.selectedMeasure;
    }

    public SongTrackCoords getSongTrackCoords() {
        return this.selectedTrack;
    }

    public MeasureComponent getSelectedComponent() {
        return this.selectedComponent;
    }

    public Duration getDuration() {
        return this.selectedDuration;
    }

    public void setSelectedDuration(Duration selectedDuration) {
        this.selectedDuration = selectedDuration;
    }

    public InstrumentString getSelectedString() {
        List strings = this.selectedTrack.getTrack().getStrings();
        Iterator it = strings.iterator();
        while (it.hasNext()) {
            InstrumentString instrumentString = (InstrumentString) it.next();
            if (instrumentString.getNumber() == this.string) {
                return instrumentString;
            }
        }
        return null;
    }

    public void moveTo(SongTrackCoords selectedTrack, MeasureCoords selectedMeasure, MeasureComponent selectedComponent) {
        this.selectedTrack = selectedTrack;
        this.selectedComponent = selectedComponent;
        this.setSelectedMeasureCoords(selectedMeasure);
        this.updatePosition();
        this.updateDuration();        
    }
    
    public void changeDuration(Duration duration){
        getMeasureCoords().changeDuration(getPosition(),duration);
        setChanges(true);
    }
    
    private void updatePosition(){
        this.position = getSelectedComponent().getStart();
    }

    
    public int getPosX(){
        return this.posX;
    }
    
    public int getPosY(){
        return this.posY;
    }    
    
    
    public boolean hasChanges() {
        return changes;
    }
    public void setChanges(boolean changes) {
        this.changes = changes;
    }
}

==> TuxGuitar-0.3.tar.gz: TuxGuitar-0.3/src/org/herac/tuxguitar/gui/tab/Caret.java <==
/*
 * Created on 30-nov-2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package org.herac.tuxguitar.gui.tab;

import java.util.Iterator;
import java.util.List;

import org.eclipse.swt.graphics.GC;
import org.herac.tuxguitar.song.managers.SongManager;
import org.herac.tuxguitar.song.models.Duration;
import org.herac.tuxguitar.song.models.InstrumentString;
import org.herac.tuxguitar.song.models.Measure;
import org.herac.tuxguitar.song.models.SongTrack;

/**
 * @author julian
 * 
 * TODO To change the template for this generated type comment go to Window - Preferences - Java - Code Style - Code Templates
 */
public class Caret {
    private Tablature tablature;
    private SongManager songManager;
    private SongCoords songCoords;
    private SongTrackCoords selectedTrack;
    private MeasureCoords selectedMeasure;
    private MeasureComponent selectedComponent;
    private Duration selectedDuration;
    private int posX;
    private int posY;
    private long position;
    private int string;
    private boolean changes;
    
    public Caret(Tablature tablature,SongManager songManager, SongCoords songCoords) {
        this.tablature = tablature;
        this.songManager = songManager;
        this.songCoords = songCoords;
        this.selectedDuration = new Duration(Duration.QUARTER);
        this.string = 1;
        this.changes = false;
    }

    private void setSelectedMeasureCoords(MeasureCoords selectedMeasure) {
        if (this.selectedMeasure != null) {
            this.selectedMeasure.setCaret(null);
        }
        this.selectedMeasure = selectedMeasure;
        if (this.selectedMeasure != null) {
            this.selectedMeasure.setCaret(this);
        }
    }

    public synchronized void update(Measure measure) {
        if(measure != null){
        SongTrack track = this.songManager.getTrack(measure);
        SongTrackCoords trackCoords = songCoords.getTrack(track);
        if(trackCoords != null){
        MeasureCoords measureCoords = trackCoords.getMeasure(measure);

        MeasureComponent component = null;
        if (getSelectedComponent() == null) {
            component = measureCoords.getFirstComponent();
        } else {
            if (getSelectedComponent() instanceof NoteCoords) {
                NoteCoords noteCoords = (NoteCoords) getSelectedComponent();
                component = measureCoords.getComponent(noteCoords.getStart(), noteCoords.getNote().getString());
            } else {
                component = measureCoords.getComponent(getSelectedComponent().getStart());
            }
        }
        if (component == null) {
            component = measureCoords.getFirstComponent();
        }

        moveTo(trackCoords, measureCoords, component);
        }
        }
    }

    public void paintCaret(GC gc, int fromX, int fromY) {
        if (selectedMeasure != null && selectedComponent != null) {
            long start = this.selectedComponent.getStart();
            this.posX = fromX + TablatureUtil.getStartPosition(selectedMeasure.getMeasure(), start, selectedMeasure.getQuarterSpan()) + this.selectedComponent.getSpan() + 4;
            this.posY = fromY + this.string * SongTrackCoords.DEFAULT_STRING_SPAN - 7;
            
            gc.drawRectangle(this.posX, this.posY, 13, 15);            
        }
        
    }

    public void setStringNumber(int string){
        this.string = string;
    }
    
    public boolean moveRight() {        
        if (this.selectedComponent != null) {
            MeasureComponent nextComponent = selectedMeasure.getNextComponent(this.selectedComponent);
            if (nextComponent != null) {
                this.selectedComponent = nextComponent;
            } else {
                //si no habia mas componentes. busco el siguiente compas
                MeasureCoords nextMeasureCoords = this.selectedTrack.getNextMeasure(this.selectedMeasure);
                if (nextMeasureCoords == null) {
                    return false;
                }else{
                    this.setSelectedMeasureCoords(nextMeasureCoords);
                    this.selectedComponent = this.selectedMeasure.getFirstComponent();
                }
            }
            this.updatePosition();
            this.updateDuration();  
            this.checkString();
            this.setChanges(true);
        }
        return true;
    }

    public void moveLeft() {
        if (this.selectedComponent != null) {
            MeasureComponent prevComponent = selectedMeasure.getPreviousComponent(this.selectedComponent);
            if (prevComponent != null) {
                this.selectedComponent = prevComponent;
            } else {
                //si no habia mas componentes. busco el compas anterior
                MeasureCoords prevMeasureCoords = this.selectedTrack.getPrevMeasure(this.selectedMeasure);
                if (prevMeasureCoords != null) {
                    this.setSelectedMeasureCoords(prevMeasureCoords);
                    this.selectedComponent = this.selectedMeasure.getLastComponent();
                }
            }
            this.updatePosition();
            this.updateDuration();
            this.checkString();
            this.setChanges(true);
        }
    }

    /**
     * Luego de mover el Caret. cambia la duracion seleccionada por la del componente. solo si lo que resta del compas no esta vacio
     */
    private void updateDuration() {
        if (this.selectedComponent != null) {
            boolean hasNotes = false;
            Iterator it = getMeasureCoords().getComponentsBeforeEnd(getSelectedComponent().getStart()).iterator();
            while (it.hasNext()) {
                MeasureComponent component = (MeasureComponent) it.next();
                if (component instanceof NoteCoords) {
                    hasNotes = true;
                    break;
                }
            }
            if (hasNotes) {
                if(this.selectedComponent instanceof SilenceCoords){
                    long length = this.selectedComponent.getDuration().getTime();
                    
                    MeasureComponent nextComponent = getMeasureCoords().getNextComponent(this.selectedComponent);
                    while(nextComponent != null && nextComponent instanceof SilenceCoords){
                        length += nextComponent.getDuration().getTime();                        
                        nextComponent = getMeasureCoords().getNextComponent(nextComponent);
                    }
                    
                    if(this.selectedDuration.getTime() > length){
                        this.selectedDuration = (Duration) this.selectedComponent.getDuration().clone();   
                    }
                }else{
                    this.selectedDuration = (Duration) this.selectedComponent.getDuration().clone();
                }
                
                //this.selectedDuration = (Duration) this.selectedComponent.getDuration().clone();                
            }
        }
    }

    public void moveUp() {
        changeString(-1);
    }

    public void moveDown() {
        changeString(1);
    }

    private void changeString(int i) {
        int nextString = this.string + i;
        int trackStrings = this.selectedTrack.getTrack().getStrings().size();
        if (nextString < 1) {
            nextString = trackStrings;
        } else if (nextString > trackStrings) {
            nextString = 1;
        }
        this.string = nextString;
    }

    public long getPosition() {
        return this.position;
    }

    public SongCoords getSongCoords() {
        return this.songCoords;
    }

    public MeasureCoords getMeasureCoords() {
        return this.selectedMeasure;
    }

    public SongTrackCoords getSongTrackCoords() {
        return this.selectedTrack;
    }

    public MeasureComponent getSelectedComponent() {
        return this.selectedComponent;
    }

    public Duration getDuration() {
        return this.selectedDuration;
    }

    public void setSelectedDuration(Duration selectedDuration) {
        this.selectedDuration = selectedDuration;
    }

    public InstrumentString getSelectedString() {
        List strings = this.selectedTrack.getTrack().getStrings();
        Iterator it = strings.iterator();
        while (it.hasNext()) {
            InstrumentString instrumentString = (InstrumentString) it.next();
            if (instrumentString.getNumber() == this.string) {
                return instrumentString;
            }
        }
        return null;
    }

    public void moveTo(SongTrackCoords selectedTrack, MeasureCoords selectedMeasure, MeasureComponent selectedComponent) {
        this.selectedTrack = selectedTrack;
        this.selectedComponent = selectedComponent;
        this.setSelectedMeasureCoords(selectedMeasure);
        this.updatePosition();
        this.updateDuration();   
        this.checkString();
    }
    
    public void changeDuration(Duration duration){
        //getMeasureCoords().changeDuration(getPosition(),duration);
        getMeasureCoords().changeDuration(getSelectedComponent(),duration);
        setChanges(true);
    }
    
    private void updatePosition(){
        this.position = getSelectedComponent().getStart();
    }
    
    private void checkString(){
        int stringCount = getSongTrackCoords().getTrack().getStrings().size();
        if(this.string > stringCount){
            this.string = stringCount;
        }
    }    
    
    public int getPosX(){
        return this.posX;
    }
    
    public int getPosY(){
        return this.posY;
    }    
    
    
    public boolean hasChanges() {
        return changes;
    }
    public void setChanges(boolean changes) {
        this.changes = changes;
    }
}
